#include "stdafx.h"

#include "system.h"
#include "Image.h"
#include "CubeLut.h"

#include <tchar.h>


// use a cube file to transform the source image
const bool _UseCubeFile( const wstring & _srcImage, const wstring & _dstImage, const wstring & _cubeFile ) {
	
	// load source image
	Image img;
	if( !img.Load( _srcImage ) ) {
		wcout << L"[ERROR] fail to load source image: '" << _srcImage << L"'\n";
		return false;
	}

	// load cube file
	CubeLut lut;
	if( !lut.Load( _cubeFile ) ) {
		wcout << L"[ERROR] fail to load cube file: '" << _cubeFile << L"'\n";
		return false;
	}

	// apply lookup table
	lut.Apply( img );

	// save transformed image
	if( !img.Save( _dstImage ) ) {
		wcout << L"[ERROR] fail to save destination image: '" << _dstImage << L"'\n";
		return false;
	}
	return true;
}


// generate a cube file by deducing transformations from source to destination images
const bool _GenCubeFile( const wstring & _srcImage, const wstring & _dstImage, const wstring & _cubeFile, const dword _dwSize ) {
	
	// load source image
	Image srcImg;
	if( !srcImg.Load( _srcImage ) ) {
		wcout << L"[ERROR] fail to load source image: '" << _srcImage << L"'\n";
		return false;
	}

	// load destination image
	Image dstImg;
	if( !dstImg.Load( _dstImage ) ) {
		wcout << L"[ERROR] fail to load destination image: '" << _dstImage << L"'\n";
		return false;
	}

	// generate a 16^3 cube lut
	if( !CubeLut::Generate( srcImg, dstImg, _cubeFile, _dwSize ) ) {
		wcout << L"[ERROR] fail to generate cube file: '" << _cubeFile << L"'\n";
		return false;
	}

	return true;
}


// main entry point
int _tmain( int argc, wchar_t * argv[] ) {

	// need at least 
	if( argc < 2 )
		goto L_WrongArgs;

	// use a cube file
	if( wstring( argv[ 1 ] ) == L"-use" ) {
		if( argc != 5 )
			goto L_WrongArgs;
		return _UseCubeFile( argv[ 2 ], argv[ 3 ], argv[ 4 ] ) ? 0 : -1;
	}
	else
	// generate a cube file
	if( wstring( argv[ 1 ] ) == L"-gen" ) {
		if( argc < 5 || argc > 6 )
			goto L_WrongArgs;
		return _GenCubeFile( argv[ 2 ], argv[ 3 ], argv[ 4 ], argc == 5 ? 17 : _wtoi( argv[ 5 ] ) ) ? 0 : -1;
	}

	// something wrong with arguments
	L_WrongArgs:
	wcout << L"usage is:\ncube -use srcimg dstimg cubefile\ncube -gen srcimg dstimg cubefile [size=17]\n";
	return -1;
}

