#pragma once

#include "system.h"

// an IRIDAS cube formatted lookup table
class CubeLut sealed {
private:
	template< class T >
	class RGB sealed {
	public:
		RGB() : r( 0 ), g( 0 ), b( 0 ) {}
		RGB( T _r, T _g, T _b ) : r( _r ), g( _g ), b( _b ) {}
		RGB & operator =( const RGB & _col ) { r = _col.r; g = _col.g; b = _col.b; return *this; }
	public:
		T r, g, b;
	};

public:
	CubeLut() : m_dwSize( 0 ) {}
	CubeLut( const dword _dwSize ) : m_dwSize( _dwSize ), m_cube( new RGB< double >[ _dwSize * _dwSize * _dwSize ] ) {}

public:
	void GenerateNoTransform();
	const bool Load( const wstring & _sFileName );
	void Apply( const class Image & _img );
	static const bool Generate( const Image & _srcImg, const Image & _dstImg, const wstring & _sFileName, const dword _dwSize );

private:
	void _Set( const dword _x, const dword _y, const dword _z, const RGB< double > & _col ) { m_cube.get()[ _x + ( _y * m_dwSize ) + ( _z * m_dwSize * m_dwSize ) ] = _col; }
	const RGB< double > & _Get( const dword _x, const dword _y, const dword _z ) const { return m_cube.get()[ _x + ( _y * m_dwSize ) + ( _z * m_dwSize * m_dwSize ) ]; }
	RGB< double > _Interpolate( const RGB< double > & _p1, const RGB< double > & _p2, const double _ratio ) { return RGB< double >( _p1.r * ( 1 - _ratio ) + _p2.r * _ratio, _p1.g * ( 1 - _ratio ) + _p2.g * _ratio, _p1.b * ( 1 - _ratio ) + _p2.b * _ratio ); }

private:
	dword _GetInt( const string & _srcData, size_t & _szPos );
	double _GetVal( const string & _srcData, size_t & _szPos );
	void _SkipTheJunk( const string & _srcData, size_t & _szPos );

private:
	dword m_dwSize;
	unique_ptr< RGB< double > > m_cube;
};