#include "stdafx.h"

#include "Image.h"
#include "jpeglib.h"


// load jpeg
const bool Image::Load( const wstring & _sFileName ) {
	
	// try to open jpeg file
	FILE * pFile;
	if( _wfopen_s( &pFile, _sFileName.c_str(), L"rb" ) != 0 ) {
		wcout << L"[ERROR] fail to read jpeg image: '" << _sFileName << L"'\n";
		return false;
	}

	// decompression info struct
	struct jpeg_decompress_struct cinfo;
	struct jpeg_error_mgr jerr;
	cinfo.err = jpeg_std_error( &jerr );

	// create decompress
	jpeg_create_decompress( &cinfo );
	jpeg_stdio_src( &cinfo, pFile );
	jpeg_read_header( &cinfo, true );
	jpeg_start_decompress( &cinfo );

	// allocate buffer
	m_dwWidth = cinfo.output_width;
	m_dwHeight = cinfo.output_height;
	if( cinfo.num_components != 3 ) {
		wcout << L"[ERROR] jpeg image '" << _sFileName << L"' is not 8-bits RGB\n";
		return false;
	}
	m_data = unique_ptr< byte >( new byte[ m_dwWidth * m_dwHeight * 3 ] );
	
	// row ptr allocation
	JSAMPROW rowPtr[ 1 ];
	unique_ptr< byte > buffRow( new byte[ m_dwWidth * 3 ] );
	rowPtr[ 0 ] = buffRow.get();

	// for each col
	dword dwPos = 0;
	while( cinfo.output_scanline < m_dwHeight ) {
		jpeg_read_scanlines( &cinfo, rowPtr, 1 );
		for( dword i = 0; i < m_dwWidth * 3; i++ )
			m_data.get()[ dwPos++ ] = rowPtr[ 0 ][ i ];
	}	

	// wrap up decompression, destroy objects, free pointers and close open files
	jpeg_finish_decompress( &cinfo );
	jpeg_destroy_decompress( &cinfo );
	fclose( pFile );
	return true;
}


// save current image as jpeg
const bool Image::Save( const wstring & _sFileName, const unsigned char _qualityPercent ) {
	Assert( _qualityPercent <= 100 );
	Assert( m_dwWidth != 0 && m_dwHeight != 0 );

	// try to create jpeg file
	FILE * pFile;
	if( _wfopen_s( &pFile, _sFileName.c_str(), L"wb+" ) != 0 ) {
		wcout << L"[ERROR] fail to create jpeg image: '" << _sFileName << L"'\n";
		return false;
	}

	// compression info struct
	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;
	cinfo.err = jpeg_std_error( &jerr );

	// create compress
	jpeg_create_compress( &cinfo );
	jpeg_stdio_dest( &cinfo, pFile );

	// set image informations
	cinfo.image_width = m_dwWidth;
	cinfo.image_height = m_dwHeight;
	cinfo.input_components = 3;
	cinfo.in_color_space = JCS_RGB;
	jpeg_set_defaults( &cinfo );
	cinfo.num_components = 3;
	cinfo.dct_method = JDCT_FLOAT;

	// set jpeg quality
	jpeg_set_quality( &cinfo, _qualityPercent, true );

	// start compression
	jpeg_start_compress( &cinfo, true );
	JSAMPROW row_pointer[ 1 ];
	while( cinfo.next_scanline < cinfo.image_height ) {
		row_pointer[ 0 ] = &m_data.get()[ cinfo.next_scanline * cinfo.image_width * 3 ];
		jpeg_write_scanlines( &cinfo, row_pointer, 1 );
	}

	// finish compression
	jpeg_finish_compress( &cinfo );
	jpeg_destroy_compress( &cinfo );
	fclose( pFile );
	return true;
}